
# --------------------------------------------------------------------------------------
# Set of CMAKE Macros/Definitions used for the managing of the documentation by Doxygen
# --------------------------------------------------------------------------------------
# Author: Loc Besnard 
# Email: Loic.Besnard@irisa.fr
# --------------------------------------------------------------------------------------

IF (CMAKE_MAJOR_VERSION VERSION_GREATER 2)
  CMAKE_POLICY(SET CMP0026 OLD)  # to be removed
  CMAKE_POLICY(SET CMP0011 OLD)  # to be removed
ENDIF(CMAKE_MAJOR_VERSION VERSION_GREATER 2)

SET(DOXYGEN_SUFFIX dox)

# Address of the data base for the cross references computations.
SET(ADDR_CROSSREF ${pK_tmp}/CrossRefs)
SET(CROSSREF_DATA_BASE_DIR ${ADDR_CROSSREF}/_DB_) # Contains a copy of all the libraries.
SET(CROSSREF_SUFFIX _CrossRefs)

# The file ${CROSSREF_DATA_BASE} is the dot file that epxresses the cross references
# between the objects of all the libraries of the Polychrony tool set (GUI not included). 
# The computation of the cross references is defined in the command $pK_ROOT/com/CrossRefs/mkCrossRefComplete.

SET(CROSSREF_DATA_BASE ${CROSSREF_DATA_BASE_DIR}/pK_ALL_OBJECTS${CROSSREF_SUFFIX}.dot)

SET_SOURCE_FILES_PROPERTIES(${CROSSREF_DATA_BASE} PROPERTIES GENERATED true)

FILE(MAKE_DIRECTORY ${CROSSREF_DATA_BASE_DIR})

IF(APPLE)
  # Instead of the command-line "doxygen" tool being placed in Doxygen.app/Contents/MacOS, "Doxywizard" is 
  # placed there instead and 
  # "doxygen" is actually placed in Contents/Resources. 
  # Once CMake sees the bundle, it indiscrimately uses Doxywizard as the executable to use.
  #So fixing the problem by disabling the app-bundle feature for only this command.
  SET(TEMP_DOXYGEN_SAVE_CMAKE_FIND_APPBUNDLE ${CMAKE_FIND_APPBUNDLE})
  # Disable the App-bundle detection feature
  SET(CMAKE_FIND_APPBUNDLE "NEVER")
  FIND_PROGRAM(DOXYGEN_EXECUTABLE
    NAMES doxygen
    DOC "where is the doxygen executable?"
    PATHS 
    $ENV{DOXYGEN_HOME}
    )
  SET(CMAKE_FIND_APPBUNDLE ${TEMP_DOXYGEN_SAVE_CMAKE_FIND_APPBUNDLE})
ELSE(APPLE)
  FIND_PROGRAM(DOXYGEN_EXECUTABLE
    NAMES doxygen
    DOC "where is the doxygen executable?"
    PATHS 
    $ENV{DOXYGEN_HOME}
    )
ENDIF(APPLE)


FIND_PROGRAM(DOXYGEN_DOT_EXECUTABLE
  NAMES dot
  DOC "where is dot executable from Graphiz (for doxygen)?"
  PATHS  
  $ENV{DOT_HOME}
  )
IF (DOXYGEN_EXECUTABLE)
  SET (DOXYGEN_FOUND TRUE)
  SET (DOXYGEN ${DOXYGEN_EXECUTABLE})
ENDIF (DOXYGEN_EXECUTABLE)

IF (DOXYGEN_DOT_EXECUTABLE)
  SET (DOXYGEN_DOT_FOUND TRUE)  
  # the directory of dot is required in doxygen.config: DOT_PATH
  GET_FILENAME_COMPONENT (DOXYGEN_DOT_EXECUTABLE_PATH ${DOXYGEN_DOT_EXECUTABLE} PATH)
  SET (DOT ${DOXYGEN_DOT_EXECUTABLE})   # deprecated
ENDIF (DOXYGEN_DOT_EXECUTABLE)

MARK_AS_ADVANCED(
  DOXYGEN_EXECUTABLE
  DOXYGEN_DOT_EXECUTABLE
  DOXYGEN_DOT_EXECUTABLE_PATH
  )

FIND_PROGRAM(CROSSREF_ANALYZER
  NAMES mkCrossRefLibrary mkCrossRefFilesofaLibrary mkCrossRefComplete
  DOC "where is the scripts for the cross references analyze?"
  PATHS ${pK_com}/CrossRefs
  )

FIND_PROGRAM(DOC_GENERATOR
  NAMES genDocRootFile
  DOC "the script used to generated FOODocRoot.dox (FOO a package)"
  PATHS ${pK_com}
  )

MACRO(COPYDOC INPUT OUTPUT)
  ADD_CUSTOM_COMMAND(
    OUTPUT ${OUTPUT}
    COMMAND ${CMAKE_COMMAND} -E echo "Copying...${OUTPUT}"
    COMMAND ${CMAKE_COMMAND} -E copy ${INPUT} ${OUTPUT}
    WORKING_DIRECTORY ${CMAKE_CURRENT_SOURCE_DIR}
    DEPENDS ${INPUT}
    )
ENDMACRO(COPYDOC INPUT OUTPUT)


# --------------------------------------------------------------------------------------------
# Command that produces the documentation using the doxygen utility.
# --------------------------------------------------------------------------------------------
MACRO(MAKEDOC DOXYGEN_INPUT DOXYGEN_OUTPUT DEP_LIST)
  FILE(TO_NATIVE_PATH ${DOXYGEN_INPUT} FOS)
  ADD_CUSTOM_COMMAND(
    OUTPUT ${DOXYGEN_OUTPUT}
    COMMAND ${CMAKE_COMMAND} -E echo "Building API Documentation..."
    COMMAND ${DOXYGEN_EXECUTABLE} ${FOS}
    COMMAND ${CMAKE_COMMAND} -E create_symlink html/index.html ${CMAKE_CURRENT_SOURCE_DIR}/doc/index.html
    COMMAND ${CMAKE_COMMAND} -E echo "Done."
    WORKING_DIRECTORY ${CMAKE_CURRENT_SOURCE_DIR}
    DEPENDS ${DEP_LIST}
    )
ENDMACRO(MAKEDOC DOXYGEN_INPUT DOXYGEN_OUTPUT DEP_LIST)

# --------------------------------------------------------------------------------------------
# Command that produces the documentation using the doxygen utility (if it is available).
# --------------------------------------------------------------------------------------------
MACRO(MAKE_DOC_TARGET DOXYGEN_INPUT DOXYGEN_OUTPUT DEP_LIST)
  IF (DOXYGEN_FOUND)
    PRINT( "DOXYGEN binary is : ${DOXYGEN_EXECUTABLE}")
    MAKEDOC( ${DOXYGEN_INPUT} ${DOXYGEN_OUTPUT} "${DEP_LIST}")
    ADD_CUSTOM_TARGET(doc DEPENDS ${DOXYGEN_OUTPUT})
  ELSE(DOXYGEN_FOUND)
    MESSAGE("Doxygen package not found.")
  ENDIF (DOXYGEN_FOUND)
ENDMACRO(MAKE_DOC_TARGET DOXYGEN_INPUT DOXYGEN_OUTPUT DEP_LIST)

# ------------------------------------------------------------------------------------------------------------------------
# It generates the files ${PACKAGE}DocDir.dox,  _DocImplPackageAbstract.dox from, respectively,
# the generic files ${pK_com}/GenericDocDir and ${pK_com}/GenericDocImplPackageAbstract.dox).
# -----
# ASSUMED: the cmake variable MOD_LIST contains the set of compiling units.
# todo: add a parameter to replace this assumption.
# ------------------------------------------------------------------------------------------------------------------------
MACRO(CONFIGURE_GENERICDOCDIR PACKAGE WHERE ADDR_DOC_SRC)
  SET(Foo ${PACKAGE})
  CONFIGURE_FILE(${pK_com}/GenericDocDir.${DOXYGEN_SUFFIX} ${WHERE}/${Foo}DocDir.${DOXYGEN_SUFFIX})
  # WARNING WARNING WARNING
  EXTRACT_FILE_NAMES("${MOD_LIST}" COMPILING_UNITS)
  CONFIGURE_FILE(${pK_com}/GenericDocImplPackageAbstract.${DOXYGEN_SUFFIX} ${WHERE}/_DocImplPackageAbstract.${DOXYGEN_SUFFIX})
ENDMACRO(CONFIGURE_GENERICDOCDIR PACKAGE WHERE ADDR_DOC_SRC)


# ------------------------------------------------------------------------------------------------------------------------
# Generating the file ${DOC_CROSS_FILE} for the package ${PACKAGE} composed of ${MOD_LIST} files.
# it generates
# /** ${PACKAGE}DocCross Cross references
#  \dotfile ${PACKAGE}.dot "cross references between the components of the package"
#  \dotfile Fi.dot "References of the Fi component", for Fi in  ${MOD_LIST}
# */
# ------------------------------------------------------------------------------------------------------------------------
MACRO(GENDOCCROSSFILE DOC_CROSS_FILE PACKAGE)
  FILE(WRITE ${DOC_CROSS_FILE} "/** \\page ${PACKAGE}DocCross Cross references\n")
  FILE(APPEND ${DOC_CROSS_FILE} " \\dotfile ${PACKAGE}.dot \"cross references between the components of the package ${PACKAGE} \"\n")
  FOREACH(VFILE ${MOD_LIST})
    GET_FILENAME_COMPONENT(ELEM ${VFILE} NAME_WE)
    FILE(APPEND ${DOC_CROSS_FILE} " \\dotfile ${ELEM}${CROSSREF_SUFFIX}.dot \"References of the ${ELEM} component\"\n")
  ENDFOREACH(VFILE)
  FILE(APPEND ${DOC_CROSS_FILE} "*/")
ENDMACRO(GENDOCCROSSFILE DOC_CROSS_FILE PACKAGE)

# --------------------------------------------------------------------------------------------------------------------------
# It produces the files ${PACKAGE}.{dot, pdf,UndefinedRef} of the documentation.
# It depends on the file ${VTARGET} (absolute path of the library); it is the package ${PACKAGE} in the directory ${EXPORT}.
# The library is assumed be available in the ${ADDR_CROSSREF}/${PACKAGE} directory.
# It assumes that the set of the modules of the package is defined in the variable MOD_LIST.
# --------------------------------------------------------------------------------------------------------------------------
MACRO(GEN_CROSS_REF VTARGETCROSSREF VTARGET PACKAGE ADDR_FIG)
  IF(CROSSREF_ANALYZER)
    SET(INPUTDIR ${ADDR_CROSSREF}/${PACKAGE})
    FILE(MAKE_DIRECTORY ${INPUTDIR})
    ADD_CUSTOM_COMMAND(
      OUTPUT ${VTARGETCROSSREF}
      COMMAND ${CMAKE_COMMAND} -E echo "mkCrossRefLibrary: Building ${VTARGETCROSSREF} from ${VTARGET}"
      COMMAND ${pK_com}/CrossRefs/mkCrossRefLibrary  ARGS ${VTARGET} ${PACKAGE}
      COMMAND ${CMAKE_COMMAND}  -E copy ${INPUTDIR}/${PACKAGE}.dot ${VTARGETCROSSREF}
#     COMMAND ${CMAKE_COMMAND}  -E copy ${INPUTDIR}/${PACKAGE}${CROSSREF_SUFFIX}.pdf ${ADDR_FIG}/${PACKAGE}${CROSSREF_SUFFIX}.pdf
      COMMAND ${CMAKE_COMMAND}  -E copy ${INPUTDIR}/${PACKAGE}.UndefinedRef ${ADDR_FIG}/${PACKAGE}.UndefinedRef
      WORKING_DIRECTORY ${INPUTDIR}
      DEPENDS ${VTARGET} 
      )
  ELSE(CROSSREF_ANALYZER)
    ADD_CUSTOM_COMMAND(
    OUTPUT ${VTARGETCROSSREF}
    COMMAND ${CMAKE_COMMAND} -E echo "Building: ${VTARGETCROSSREF} from ${VTARGET}: NOT BUILT"
    WORKING_DIRECTORY ${INPUTDIR}
    DEPENDS ${VTARGET}
    )
ENDIF(CROSSREF_ANALYZER)
ENDMACRO(GEN_CROSS_REF VTARGETCROSSREF VTARGET PACKAGE ADDR_FIG)

# It generates for each file F of a library (element of the MOD_LIST variable, assumed set).
# the cross references Fi_CrossRefs.dot, (at one level) of this file.
# It is assumed that the  ${CROSSREF_DATA_BASE} file is built.
MACRO(ADD_GENERATED_DOT_TO_DEP_LIST ROOTFILEDOC_DEPENDS VTARGET PACKAGE ADDR_FIG)
  IF(CROSSREF_ANALYZER)
    SET(L ${ROOTFILEDOC_DEPENDS})
    FOREACH(VFILE ${MOD_LIST})
      GET_FILENAME_COMPONENT(ELEM_${VFILE} ${VFILE} NAME_WE)
      SET(FNAME ${ELEM_${VFILE}}${CROSSREF_SUFFIX}.dot)
      SET(FF ${ADDR_FIG}/${FNAME})
      SET(L ${L} ${FF})
      SET(INPUTDIR ${CROSSREF_DATA_BASE_DIR})
      SET(SRC ${INPUTDIR}/${FNAME})
      SET_SOURCE_FILES_PROPERTIES(${SRC} PROPERTIES GENERATED true)
      # Producing the cross references for the files of a library, in the ${CROSSREF_DATA_BASE_DIR} directory.
      # MESSAGE("mkCrossRefFilesofaLibrary ${VTARGET}, it produces ${SRC}")
      ADD_CUSTOM_COMMAND(
	OUTPUT ${SRC}
	COMMAND ${CMAKE_COMMAND} -E echo "mkCrossRefFilesofaLibrary: Building  ${SRC} from ${VTARGET}"
	COMMAND ${pK_com}/CrossRefs/mkCrossRefFilesofaLibrary ARGS ${VTARGET}
	COMMAND ${CMAKE_COMMAND}  -E copy  ${SRC} ${FF}
	WORKING_DIRECTORY ${CROSSREF_DATA_BASE_DIR}
#	DEPENDS ${CROSSREF_DATA_BASE}
	DEPENDS  ${VTARGET}
	)
      # copy of the src file to the doc directory.
      ADD_CUSTOM_COMMAND(
	OUTPUT ${FF}
	COMMAND ${CMAKE_COMMAND}  -E copy  ${SRC} ${FF}
	WORKING_DIRECTORY ${CROSSREF_DATA_BASE_DIR}
	DEPENDS ${SRC}
	)
    ENDFOREACH(VFILE)

    SET(ROOTFILEDOC_DEPENDS "${L}")
  ENDIF(CROSSREF_ANALYZER)
ENDMACRO(ADD_GENERATED_DOT_TO_DEP_LIST ROOTFILEDOC_DEPENDS VTARGET PACKAGE ADDR_FIG)

# It generates the ${ROOTFILEDOC} file using the script ${pK_com}/genDocRootFile.
# The variable PACKAGE_HEADER_DOC contains the title associated with the ${PACKAGE}DocRoot.dox file.
#If a file ${PACKAGE}DocRoot_Manual.dox exists, this file is copied in the ${PACKAGE}DocRoot.dox file.
MACRO(GEN_DOCROOTFILE PACKAGE ROOTFILEDOC WHERE ROOTFILEDOC_DEPENDS PACKAGE_HEADER_DOC)
  IF(DOC_GENERATOR)
    SET(MANUAL_FILE ${WHERE}/${PACKAGE}DocRoot_Manual.dox)
    IF (EXISTS ${MANUAL_FILE})
       SET(DOC_ROOT_FILE ${WHERE}/${PACKAGE}DocRoot.dox)
       ADD_CUSTOM_COMMAND(
	OUTPUT ${ROOTFILEDOC}
	COMMAND ${CMAKE_COMMAND} -E echo "Copying ${MANUAL_FILE} in ${DOC_ROOT_FILE}"
	COMMAND ${CMAKE_COMMAND}  -E copy ${MANUAL_FILE}  ${DOC_ROOT_FILE}
	DEPENDS ${pK_com}/genDocRootFile ${MANUAL_FILE}
	)
    ELSE (EXISTS ${MANUAL_FILE})      
      ADD_CUSTOM_COMMAND(
	OUTPUT ${ROOTFILEDOC}
	COMMAND ${CMAKE_COMMAND} -E echo "genDocRootFile: Generating ${ROOTFILEDOC}"
	COMMAND ${pK_com}/genDocRootFile ARGS ${PACKAGE} ${PACKAGE_HEADER_DOC}
	WORKING_DIRECTORY ${WHERE}
	DEPENDS ${pK_com}/genDocRootFile ${ROOTFILEDOC_DEPENDS}
	)
    ENDIF(EXISTS ${MANUAL_FILE})      
  ELSE(DOC_GENERATOR)
    ADD_CUSTOM_COMMAND(
      OUTPUT ${ROOTFILEDOC}
      COMMAND ${CMAKE_COMMAND} -E  echo "touching the file ${ROOTFILEDOC}"
      COMMAND  ${CMAKE_COMMAND} -E touch ${ROOTFILEDOC}
      WORKING_DIRECTORY ${WHERE}
      DEPENDS ${ROOTFILEDOC_DEPENDS}
      )
  ENDIF(DOC_GENERATOR)
ENDMACRO(GEN_DOCROOTFILE PACKAGE ROOTFILEDOC WHERE ROOTFILEDOC_DEPENDS PACKAGE_HEADER_DOC)

MACRO(MK_DOC_PACKAGE PACKAGE SRC EXPORT ADDR_DOC ADDR_DOC_SRC ADDR_DOC_HTML PACKAGE_HEADER_DOC)

  CONFIGURE_GENERICDOCDIR(${PACKAGE} ${ADDR_DOC_SRC} ${ADDR_DOC_SRC})
  SET(DOC_CROSS_FILE ${ADDR_DOC_SRC}/${PACKAGE}DocCross.${DOXYGEN_SUFFIX})
  SET(DOC_FIGDIR ${ADDR_DOC}/fig)
  
  GENDOCCROSSFILE(${DOC_CROSS_FILE} ${PACKAGE})
  SET(DOXYGEN_INPUT ${ADDR_DOC}/doxygen${PACKAGE})
  SET(DOXYGEN_OUTPUT ${ADDR_DOC_HTML}/index.html)
  
  SET(ROOTFILEDOC ${ADDR_DOC_SRC}/${PACKAGE}DocRoot.${DOXYGEN_SUFFIX})
  SET_SOURCE_FILES_PROPERTIES(${ROOTFILEDOC} PROPERTIES GENERATED true)
  SET(DEP_LIST ${ROOTFILEDOC} ${DOXYGEN_INPUT})
  
  SET(ROOTFILEDOC_DEPENDS
    ${ADDR_DOC_SRC}/_DocImplPackageAbstract.${DOXYGEN_SUFFIX}
    ${ADDR_DOC_SRC}/${PACKAGE}DocIntro.${DOXYGEN_SUFFIX}
    ${ADDR_DOC_SRC}/${PACKAGE}DocImpl.${DOXYGEN_SUFFIX} 
    ${ADDR_DOC_SRC}/${PACKAGE}DocDir.${DOXYGEN_SUFFIX} 
    ${DOC_CROSS_FILE}
    )
  
  IF(LIBRARY_OUTPUT_PATH)
    GET_TARGET_PROPERTY(VTARGET_${PACKAGE} ${PACKAGE} LOCATION)
    SET(VTARGETCROSSREF ${DOC_FIGDIR}/${PACKAGE}.dot)
    SET(ROOTFILEDOC_DEPENDS ${ROOTFILEDOC_DEPENDS}  ${VTARGET_${PACKAGE}} ${VTARGETCROSSREF})
    SET_SOURCE_FILES_PROPERTIES(${DOC_CROSS_FILE} OBJECT_DEPENDS ${VTARGETCROSSREF})
  ENDIF(LIBRARY_OUTPUT_PATH)
  
  MAKE_DOC_TARGET(${DOXYGEN_INPUT} ${DOXYGEN_OUTPUT} "${DEP_LIST}")
  IF (LIBRARY_OUTPUT_PATH)
    ADD_GENERATED_DOT_TO_DEP_LIST("${ROOTFILEDOC_DEPENDS}" ${VTARGET_${PACKAGE}} ${PACKAGE} ${DOC_FIGDIR})
    GEN_DOCROOTFILE(${PACKAGE} ${ROOTFILEDOC} ${ADDR_DOC_SRC} "${ROOTFILEDOC_DEPENDS}" ${PACKAGE_HEADER_DOC})
    GEN_CROSS_REF(${VTARGETCROSSREF} ${VTARGET_${PACKAGE}} ${PACKAGE} ${DOC_FIGDIR} )
  ELSE (LIBRARY_OUTPUT_PATH)
    GEN_DOCROOTFILE(${PACKAGE} ${ROOTFILEDOC} ${ADDR_DOC_SRC} "${ROOTFILEDOC_DEPENDS}" ${PACKAGE_HEADER_DOC})
  ENDIF(LIBRARY_OUTPUT_PATH)
  
ENDMACRO(MK_DOC_PACKAGE PACKAGE SRC EXPORT ADDR_DOC ADDR_DOC_SRC ADDR_DOC_HTML PACKAGE_HEADER_DOC)

MACRO(MK_DOC_PACKAGE_NO_SRC PACKAGE ADDR_DOC ADDR_DOC_SRC ADDR_DOC_HTML )
  SET(EMPTYSTR)
  MK_DOC_PACKAGE(${PACKAGE} "${EMPTYSTR}" "${EMPTYSTR}" ${ADDR_DOC} ${ADDR_DOC_SRC} ${ADDR_DOC_HTML} " ")
ENDMACRO(MK_DOC_PACKAGE_NO_SRC PACKAGE ADDR_DOC ADDR_DOC_SRC ADDR_DOC_HTML )

# ---------------------------------------
# Adding "NO ACTION" for the doc target.
# ---------------------------------------
MACRO(MK_EMPTY_DOC_TARGET )
  ADD_CUSTOM_TARGET(doc )
ENDMACRO(MK_EMPTY_DOC_TARGET )

# -------------------------------------------------------------
# It adds the dependence DIRFROM/index.html to DIRTO/index.html
# -------------------------------------------------------------
MACRO(ADD_DOC_DEPENDENCE DIRTO DIRFROM)
  SET_SOURCE_FILES_PROPERTIES( ${DIRTO}/index.html PROPERTIES GENERATED true OBJECT_DEPENDS ${DIRFROM}/index.html )
ENDMACRO(ADD_DOC_DEPENDENCE DIRTO DIRFROM)

# -------------------------------
# Target "doc" for the examples
# -------------------------------
MACRO(MK_DOC_EXAMPLE PACKAGE ADDR_DOC ADDR_DOC_SRC ADDR_DOC_HTML )
#  CONFIGURE_GENERICDOCDIR(${PACKAGE} ${ADDR_DOC_SRC} ${ADDR_DOC_SRC})
  SET(DOXYGEN_INPUT ${ADDR_DOC}/doxygen${PACKAGE})
  SET(DOXYGEN_OUTPUT ${ADDR_DOC_HTML}/index.html)
  SET(DEP_LIST ${DOXYGEN_INPUT}
    ${ADDR_DOC_SRC}/${PACKAGE}DocRoot.${DOXYGEN_SUFFIX}
    )
  MAKE_DOC_TARGET(${DOXYGEN_INPUT} ${DOXYGEN_OUTPUT} "${DEP_LIST}")
ENDMACRO(MK_DOC_EXAMPLE PACKAGE ADDR_DOC ADDR_DOC_SRC ADDR_DOC_HTML )

# ----------------------------------------------------------------------------------------------
# Command that produces the ${FILEWITHOUTSUFF}.pdf file from the dot file ${FILEWITHOUTSUFF}.dot
# WHERE: the working directory assumed to be the directory of ${FILEWITHOUTSUFF}.dot
# ----------------------------------------------------------------------------------------------
MACRO(DOT2PDF FILEWITHOUTSUFF  WHERE)
  SET(INPUT  ${FILEWITHOUTSUFF}.dot)
  SET(OUTPUT ${FILEWITHOUTSUFF}.pdf)
  ADD_CUSTOM_COMMAND(
    OUTPUT ${OUTPUT}
    COMMAND ${CMAKE_COMMAND} -E echo "Building  ${OUTPUT}.pdf from ${INPUT} "
    COMMAND dot ARGS -Tpdf ${INPUT} -o ${OUTPUT}
    WORKING_DIRECTORY ${WHERE}
    DEPENDS ${INPUT}
    )
ENDMACRO(DOT2PDF FILEWITHOUTSUFF WHERE)
