#!/usr/bin/tcsh

# Generating distributed code for the equationSolving.SIG  program (annotated by runOn pragma).
# synoptic:  demo.sh [param]
#     Called with a parameter, the script is executed in interactive mode. (no test on the value of this parameter).

setenv INTER_MODE TRUE
if ( "$#" == 0 ) then
    setenv INTER_MODE FALSE
endif

setenv HERE $PWD
setenv AppliName equationSolving
setenv DataDir ${HERE}/${AppliName}/data

if ( ! $?Signal_bin ) then 
    echo "##  ERROR  ## Signal_bin not defined"
    echo "     run the Polychrony setup, please."
    exit
 endif



# Compiling for code distribution
# -------------------------------
$Signal_bin/signal -TRA -c ${AppliName}.SIG -dist

if ("$INTER_MODE" == "TRUE" ) then
    echo "Continue[Y/N]?"
    setenv X $<
    if ( "$X" == "N" ) then
	exit
    endif
endif

# It will be necessary to protect the access to the gtkwave file (semaphore): one writer at a time.
# echo " Do you want a visualization of the results by gtkWave[Y/N]?"
#setenv GTKWAVE $<
setenv GTKWAVE "N"

# Compiling the code generation (first part)
# ------------------------------------------
cd equationSolving/equationSolving_1
if (! -efr "runConfig.h") then
    cp ${DataDir}/runConfig.h .
 endif
if (! -efr "Ra.dat") then
    cp ${DataDir}/R*.dat .
endif
if ( "$GTKWAVE" == "Y" ) then
    ed equationSolving_1_io.c <<EOF
/_DUMPFIL/
a
#define _DUMPFILE_VCD_
.
w
q
EOF

endif
$Signal_bin/genMake C equationSolving_1
make dist

# Compiling the code generation (second part)
# ------------------------------------------
cd $HERE
cd equationSolving/equationSolving_2 
if (! -efr "runConfig.h") then
    cp ${DataDir}/runConfig.h .
endif
if (! -efr "Ra.dat") then
    cp ${DataDir}/R*.dat .
endif
if ( "$GTKWAVE" == "Y" ) then
    ed equationSolving_2_io.c <<EOF
/_DUMPFIL/
a
#define _DUMPFILE_VCD_
.
w
q
EOF

endif
$Signal_bin/genMake C equationSolving_2
make dist

cd $HERE
if ("$INTER_MODE" == "TRUE" ) then
    echo " Continue[Y/N]?"
    setenv X $<
    if ( "$X" == "N" ) then
	exit
    endif
endif

# Simulation using MPICH or openMPI: for MPICH it is ok, not for openMPI (July 2013).

mpiexec -np 1 -wdir equationSolving/equationSolving_1 equationSolving_1 : -np 1 -wdir equationSolving/equationSolving_2 equationSolving_2 &

sleep 2
killall mpiexec

if (! -efr "equationSolving/equationSolving_1/Woc.dat") then
   echo " >>>>>>>>ERROR: The file equationSolving/equationSolving_1/Woc.dat not generated!!!"
    goto endlab
endif

if (! -efr "equationSolving/equationSolving_1/Wx1.dat") then
   echo " >>>>>>>>ERROR: The file equationSolving/equationSolving_1/Wx1.dat not generated!!!"
    goto endlab
endif

if (! -efr "equationSolving/equationSolving_1/Wx2.dat") then
   echo " >>>>>>>>ERROR: The file equationSolving/equationSolving_1/Wx2.dat not generated!!!"
    goto endlab
endif


echo
echo "------°------°------°------°------°------°------°------°------°------°------°"
echo "The results are produced in equationSolving/equationSolving_1/W*.dat files."
echo "The awaited results are in equationSolving/data/W*.ok files."
echo "------°------°------°------°------°------°------°------°------°------°------°"
echo " -- Begin diff <<<<"

diff equationSolving/equationSolving_1/Woc.dat ${DataDir}/Woc.ok
diff equationSolving/equationSolving_1/Wx1.dat ${DataDir}/Wx1.ok
diff equationSolving/equationSolving_1/Wx2.dat ${DataDir}/Wx2.ok
echo " -- End  diff  >>>>"

if ( "$GTKWAVE" == "Y" ) then
echo " (gtkwave) Continue[Y/N]?"
setenv X $<
if ( "$X" == "N" ) then
     exit
endif

gtkwave equationSolving/equationSolving_1/W_DUMPFILE_VCD_.dat
endif

endlab:
    cd $HERE

