
import java.io.*;
import SignalLibraries.Java.graphical.*;

/**
   This class builts a window for the simulation (with button, canvas, ...) of a Signal Program
   when C is produced by the Signal compiler and when the predefined provided libGraphics.SIG 
   Signal library is used in the Signal program.
   To make portable the simulation program, the graphical part is written in Java.
   The connexion Java/C is done through SimuEquationSolving.c module defined in export directory.
*/
public class SimuEquationSolving extends SimuC
{
    public static final int width=80;
    public static final int heigth=30;

    private int frame;
    private int strA, strB, strC;
    private int  reset, go, end;
    private int X1, X2, stepWidg, other;

    private boolean no_communication;
    private int nb_steps;
    private float v_A, v_B, v_C;
    
    private void mkGraphicalWindows()
    {
	int Ipanel, Ipanel1, Opanel, Opanel1;

	frame = gw.createFrame(500,300);
	
	Ipanel = gw.createPanel(frame, Direction.vertic);

	Ipanel1 = gw.createPanel(Ipanel, Direction.horiz);
	gw.createLabel(Ipanel1);
	Ipanel1 = gw.createPanel(Ipanel, Direction.horiz);

	strA = gw.createTextAreaEditable(Ipanel1, width, heigth);
	strB = gw.createTextAreaEditable(Ipanel1, width, heigth);
	strC = gw.createTextAreaEditable(Ipanel1, width, heigth);

	Ipanel1 = gw.createPanel(Ipanel, Direction.horiz);
	reset = gw.createButton(Ipanel1);
	go = gw.createButton(Ipanel1);
	end = gw.createButton(Ipanel1);

	Opanel = gw.createPanel(Ipanel, Direction.horiz);
	gw.createLabel(Opanel);
	Opanel1 = gw.createPanel(Ipanel, Direction.horiz);

	X1 = gw.createTextArea(Opanel1, width, heigth);
	X2 = gw.createTextArea(Opanel1,  width, heigth);
        other = gw.createTextArea(Opanel1, width, heigth);
	stepWidg = gw.createTextArea(Opanel1, width/2 , heigth);

    }

    private void display_nbsteps()
    {
	if (nb_steps == 0) gw.setString(stepWidg, ""); else gw.setTextArea(stepWidg, nb_steps);
    }

	    
    private void reset_action()
    {
	clearInputs();
	clearOutputs();
	no_communication = true;
	nb_steps = 0;
	display_nbsteps();
    }


    private void end_action()
    {
	gw.quitAppli();
    }


    private void clearOutputs()
    {
	gw.setString(X1, "");
	gw.setString(X2, "");
	gw.setString(other, "      ");
    }

    private void clearInputs()
    {
	gw.setString(strA, "");
	gw.setString(strB, "");
	gw.setString(strC, "");
    }


    private void go_action()
    {
	getInputs();
	clearOutputs();
	
	no_communication = true;
	nb_steps = 1;
	display_nbsteps();
	
	// Signal iterating
	iterateSignal();
	while (no_communication) 
	    {
		nb_steps ++;
		display_nbsteps();
		iterateSignal();
	    }
	// clearInputs();	
    }

    private float getInput(int widg)
    {
	String str;
	str = gw.getString(widg);
	if (str.equals("")) return (float)0; return Float.parseFloat(str);
    }
	
    public void getInputs()
    {
	v_A = getInput(strA);
	v_B = getInput(strB);
	v_C = getInput(strC);
	// System.out.println(" A : " + v_A + " B: " + v_B + " C: " + v_C);
    }

    public float getA() { return v_A; }
    public float getB() { return v_B; }
    public float getC() { return v_C; }

    public void  setX1(float x) 
    {
	no_communication = false;
	gw.setTextAreaFloat(X1,x);
    }
    
    public void  setX2(float x) 
    {
	no_communication = false;
	gw.setTextAreaFloat(X2,x);
    }

    public void  setInfiniteOrNoSolution(boolean b) 
    {
	no_communication = false;
	if (b) 
	    gw.setString(other, "no solution");
	else
	    gw.setString(other, "infinite");
    }
    
    public void running()
    {
	initSimu();
	while(true)
	    {
		try{
		    Thread.currentThread().sleep(100);//sleep for 100 ms
		    if (readButton(end)) end_action();
		    if (readButton(reset)) reset_action();
		    if (readButton(go)) go_action();
		}
		catch(Exception e)
		    {
			System.out.println(e);
		    }
	    }
    }

    public SimuEquationSolving ()
    {

	mkGraphicalWindows();
	running();
	
    }



   public static void main ( String[] args )  throws Exception 
    {
	new SimuEquationSolving( );
    }

}



