/* 
   Authors: Loic Besnard, Patricia Bournai, Thierry Gautier, Paul Le Guernic.
   Copyright (C) by INRIA-CNRS - developed by Espresso project - GPL v2 License - 2010.

   Contact: polychronycontact@listes.irisa.fr 

   This file is part of Polychrony Signal Toolbox (http://www.irisa.fr/espresso/polychrony).

   Polychrony  Signal Toolbox is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License Version 2 as published by the Free
   Software Foundation.

   Polychrony Signal Toolbox is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License Version 2
   for more details.

   You should have received a copy of the GNU General Public License Version 2
   along with Polychrony Signal Toolbox; see the file COPYING. If not, consult the url 
    http://www.gnu.org/licenses/gpl-2.0.html, or write to the Free
   Software Foundation, 51 Franklin Street, Fifth Floor, Boston, MA
   02110-1301, USA. 
*/
#ifndef __SAMPLINGPORTMANAGER_H__
#define __SAMPLINGPORTMANAGER_H__



/**********************************************************************************/
/****************************  SAMPLING PORT OBJECT : BEGIN *************************/
/**********************************************************************************/

/* Object representing APEX SamplingPort */
typedef class SamplingPort * ptSamplingPort;
class SamplingPort
{
   public :
   APEX_SamplingPort_type samplingport;
   SamplingPort(APEX_SamplingPort_type s);
};

/**********************************************************************************/
/*****************************  SAMPLING PORT OBJECT : END   ************************/
/**********************************************************************************/




/**********************************************************************************/
/*********************** SAMPLING PORT MANAGER_NODE  : BEGIN   ***********************/
/**********************************************************************************/

/* Type of manager node contents */
struct SamplingPortAssoc_type
{
  Comm_ComponentID_type SamplingPort_ID; 
  ptSamplingPort ptrSamplingPort;
};

/* Manager node type definition */
typedef class SamplingPortManagerNode *ptSamplingPortManagerNode;
class SamplingPortManagerNode
{
 private :
  SamplingPortAssoc_type Contents;                             /* Current node contents */
  ptSamplingPortManagerNode next;                                     /* Next node */

 public :
  SamplingPortManagerNode();
  SamplingPortManagerNode(SamplingPortAssoc_type sampling_port);
  ~SamplingPortManagerNode();
  void setContents(SamplingPortAssoc_type contents);
  SamplingPortAssoc_type getContents();
  void setNext(ptSamplingPortManagerNode node);
  ptSamplingPortManagerNode getNext();
};

/**********************************************************************************/
/************************ SAMPLING PORT MANAGER_NODE  : END   ***********************/
/**********************************************************************************/




/**********************************************************************************/
/************************** SAMPLING PORT MANAGER : BEGIN  *************************/
/**********************************************************************************/

/* Manager type definition */
typedef class SamplingPortManager *ptSamplingPortManager;
class SamplingPortManager
{
 private :
  ptSamplingPortManagerNode first;                    /* First node in the manager */
  ptSamplingPortManagerNode last;                      /* Last node in the manager */
  int max_size;                                       /* Size limit of the manager */
  int current_size;                          /* Current number of nodes in manager */
  bool created;
  bool getElemNamed(Comm_ComponentName_type name, ptSamplingPortManagerNode *elem);
  bool getElemId( Comm_ComponentID_type samplingport_ID, ptSamplingPortManagerNode *elem);
  void setFirst(ptSamplingPortManagerNode node);
  void setLast(ptSamplingPortManagerNode node);
  ptSamplingPortManagerNode getFirst();
  ptSamplingPortManagerNode getLast();

 public :
  SamplingPortManager();
  SamplingPortManager(int size);
  ~SamplingPortManager();


 bool SamplingPortRecord(Comm_ComponentName_type samplingport_Name,
			 SamplingPortSize_type samplingPort_Size, 
			 PortDirection_type port_direction, 
			 SystemTime_type refresh_period, 
			 Comm_ComponentID_type& samplingport_ID); 
 /* If the manager was not full, a new sampling port node is created, initialized
     with input parameters and it is returned true. Else, it is returned false. */

  bool SamplingPortCheckName(Comm_ComponentName_type samplingport_Name, 
			     Comm_ComponentID_type& samplingport_ID); 
  /* If a blackboard named samplingport_Name exists in the manager, samplingport_ID
     is its identifier and it is returned true. Else, it is returned false */
  
  bool SamplingPortCheckID(Comm_ComponentID_type samplingport_ID, 
			   APEX_SamplingPort_type& samplingport);
  /* If there is an identifier samplingport_ID in the manager, board is the 
     samplingport associated and it is returned true. Else, it is returned false */
  
  void SamplingPortUpdate(APEX_SamplingPort_type samplingport);
  /* The sampling port from the manager, which has the same name as samplingport, is 
     updated */
  
  bool Full();
  /* Returns true if the manager size limit is reached. Else, it is returned
     false */
  
  bool SamplingPortCheckCreated(Comm_ComponentName_type samplingport_Name);
  
  void SamplingPortSetCreated(Comm_ComponentName_type samplingport_Name);
  
  void printFile(Comm_ComponentID_type samplingport_ID, char *result_file);
  /* Prints samplingport_ID status in result_file */
  
};

/*********************************************************************************/
/********************************* SAMPLING PORT MANAGER : END  ******************************/
/*********************************************************************************/


#endif // __SAMPLINGPORTMANAGER_H__










