#include "TYPES.h"

#include "FIFO_SendQueue.h"
#include "FIFO_ReceiveQueue.h"
#include "PRIORITY_SendQueue.h"
#include "PRIORITY_ReceiveQueue.h"

#include "BufferManager.h"


/*****************************************   BUFFER OBJECT  *********************************************/

Buffer::Buffer (APEX_Buffer_type * b):buffer (b)
{
}


/***************************************  BUFFER MANAGER_NODE  *****************************************/

BufferManagerNode::BufferManagerNode (BuffAssoc_type value):next (NULL)
{
  contents.BufferID = value.BufferID;
  contents.ptrBuffer = value.ptrBuffer;
}

BufferManagerNode::BufferManagerNode ()
{
}

BufferManagerNode::~BufferManagerNode ()
{
}

void
BufferManagerNode::setValue (BuffAssoc_type value)
{
  contents.BufferID = value.BufferID;
  contents.ptrBuffer = value.ptrBuffer;
}

void
BufferManagerNode::setNext (ptBufferManagerNode ptvalue)
{
  next = ptvalue;
}

BuffAssoc_type
BufferManagerNode::getValue ()
{
  return contents;
}

ptBufferManagerNode
BufferManagerNode::getNext ()
{
  return next;
}



/*************************************  BUFFER MANAGER  **********************************/

BufferManager::BufferManager (int size):first (NULL), last (NULL), max_size (size), current_size (0)
{
}

BufferManager::BufferManager ():first (NULL), last (NULL), max_size (MAX_NUMBER_OF_BUFFERS), current_size (0)
{
}

BufferManager::~BufferManager ()
{
}

void
BufferManager::setFirst (ptBufferManagerNode ptvalue)
{
  first = ptvalue;
}

void
BufferManager::setLast (ptBufferManagerNode ptvalue)
{
  last = ptvalue;
}

ptBufferManagerNode
BufferManager::getFirst ()
{
  return first;
}

ptBufferManagerNode
BufferManager::getLast ()
{
  return last;
}



bool
BufferManager::BufferRecord (Comm_ComponentName_type buffer_Name,
			     int buffer_Size, int message_Size, QueuingDiscipline_type queuing_Discipline, Comm_ComponentID_type & bufferID)
{

  if (current_size >= max_size)
    return pk_false;
  current_size++;
  BuffAssoc_type *new_record;
  new_record = new BuffAssoc_type ();
  new_record->BufferID = current_size;

  APEX_Buffer_type *buffer;
  buffer = new APEX_Buffer_type ();
  strcpy (buffer->Buffer_Name, buffer_Name);
  buffer->Nb_Message = 0;
  buffer->Message_Size = message_Size;
  buffer->Max_Message = buffer_Size;
  buffer->WaitingOnSend_Processes = 0;
  buffer->WaitingOnReceive_Processes = 0;
  buffer->Queuing_Discipline = queuing_Discipline;
  if (queuing_Discipline == FIFO)
    {
      buffer->WaitingOnSend_Queue = new FIFO_SendQueue ();
      buffer->WaitingOnReceive_Queue = new FIFO_ReceiveQueue ();
    }
  else
    {
      buffer->WaitingOnSend_Queue = new PRIORITY_SendQueue ();
      buffer->WaitingOnReceive_Queue = new PRIORITY_ReceiveQueue ();
    };

  ptBuffer ptsem = new Buffer (buffer);	// inutilise
  new_record->ptrBuffer = new Buffer (buffer);

  bufferID = current_size;
  ptBufferManagerNode new_node = new BufferManagerNode (*new_record);
  if (first == NULL)
    {
      first = new_node;
      last = new_node;
    }
  else
    {
      ptBufferManagerNode iter = first;
      while (iter != last)
	{
	  iter = iter->getNext ();
	};
      if (iter == last)
	{
	  last->setNext (new_node);
	  last = new_node;
	}
    };
  return pk_true;
}


bool
BufferManager::BufferCheckName (Comm_ComponentName_type bufferName, Comm_ComponentID_type & bufferID)
{
  bool retour;
  if (first == NULL)
    {
      bufferID = NULL;
      retour = pk_false;
    }
  else
    {
      ptBufferManagerNode iter = first;
      while (strcmp ((((iter->getValue ()).ptrBuffer)->buffer)->Buffer_Name, bufferName) && (iter != last))
	{
	  iter = iter->getNext ();
	};

      if (iter == last)
	{
	  if (strcmp ((((iter->getValue ()).ptrBuffer)->buffer)->Buffer_Name, bufferName))
	    {
	      bufferID = NULL;
	      retour = pk_false;
	    }
	  else
	    {
	      bufferID = (iter->getValue ()).BufferID;
	      retour = pk_true;
	    }
	}
      else
	{
	  bufferID = (iter->getValue ()).BufferID;
	  retour = pk_true;
	}
    };
  return retour;
}

bool
BufferManager::BufferCheckID (Comm_ComponentID_type bufferID, APEX_Buffer_type & buffer)
{
  bool retour;
  if (first == NULL)
    {
      retour = pk_false;
    }
  else
    {
      ptBufferManagerNode iter = first;
      while (((iter->getValue ()).BufferID != bufferID) && (iter != last))
	{
	  iter = iter->getNext ();
	};
      if (iter == last)
	{
	  if ((iter->getValue ()).BufferID != bufferID)
	    {
	      retour = pk_false;
	    }
	  else
	    {
	      buffer = *((last->getValue ()).ptrBuffer)->buffer;
	      retour = pk_true;
	    }
	}
      else
	{
	  buffer = *((iter->getValue ()).ptrBuffer)->buffer;
	  retour = pk_true;
	}
    };
  return retour;
}

void
BufferManager::BufferUpdate (APEX_Buffer_type buffer)
{
  int b;
  APEX_Buffer_type *x;

  return;

  fprintf (stderr, "BufferUpdate \n");
  fflush (stderr);
  if (first != NULL)
    {
      ptBufferManagerNode iter = first;
      Comm_ComponentName_type name;
      strcpy (name, buffer.Buffer_Name);
      while ((strcmp ((((iter->getValue ()).ptrBuffer)->buffer)->Buffer_Name, name) != 0) && (iter != last))
	iter = iter->getNext ();
      if (iter != last)
	b = pk_true;
      else
	b = (strcmp ((((iter->getValue ()).ptrBuffer)->buffer)->Buffer_Name, name) == 0);
      if (b)
	{
	  x = (((iter->getValue ()).ptrBuffer)->buffer);	// warning : side effect on the selected buffer.
	  for (int i = 0; i < MAX_BUFFER_SIZE; i++)
	    x->Message_Range[i] = buffer.Message_Range[i];
	  x->Nb_Message = buffer.Nb_Message;
	  x->Max_Message = buffer.Max_Message;
	  x->WaitingOnSend_Processes = buffer.WaitingOnSend_Processes;
	  x->WaitingOnReceive_Processes = buffer.WaitingOnReceive_Processes;
	  x->Queuing_Discipline = buffer.Queuing_Discipline;
	  x->WaitingOnSend_Queue = buffer.WaitingOnSend_Queue;
	  x->WaitingOnReceive_Queue = buffer.WaitingOnReceive_Queue;
	}
    }
}

bool
BufferManager::Full ()
{
  return (max_size == current_size);
}


void
BufferManager::printFile (Comm_ComponentID_type bufferID, char *result_file)
{
  FILE *file_out;
  APEX_Buffer_type buff;

  if (this->BufferCheckID (bufferID, buff))
    {
      file_out = fopen (result_file, "wt");
      if (!file_out)
	{
	  fprintf (stderr, "Can not open file %s\n", result_file);
	  exit (1);
	};
      fprintf (file_out,
	       "(Buffer_Identifier : %i, Name : %s, Nb_Message : %i, Max_Message : %i, WaitingOnSend_Processes  : %i, WaitingOnReceive_Processes : %i, Queuing_Discipline : %i) \n",
	       bufferID, buff.Buffer_Name, buff.Nb_Message, buff.Max_Message, buff.WaitingOnSend_Processes, buff.WaitingOnReceive_Processes,
	       buff.Queuing_Discipline);

      fclose (file_out);
    }
}
