#include "TYPES.h"
#include "FIFO_ReceiveQueue.h"



/*****************************************************************************************************/
/*************************************  FIFO_ReceiveQueue_NODE  **************************************/
/*****************************************************************************************************/



FIFO_ReceiveQueueNode::FIFO_ReceiveQueueNode ()
{
}

FIFO_ReceiveQueueNode::FIFO_ReceiveQueueNode (ProcessID_type Process_ID)
{
  contents = Process_ID;
  next = NULL;
}

void
FIFO_ReceiveQueueNode::setContents (ProcessID_type Process_ID)
{
  contents = Process_ID;
}

void
FIFO_ReceiveQueueNode::setNext (ptFIFO_ReceiveQueueNode nextNode)
{
  next = nextNode;
}

ProcessID_type
FIFO_ReceiveQueueNode::getContents ()
{
  return contents;
}

ptFIFO_ReceiveQueueNode
FIFO_ReceiveQueueNode::getNext ()
{
  return next;
}



/****************************************************************************************************/
/*************************************** FIFO_ReceiveQueue   ****************************************/
/****************************************************************************************************/


FIFO_ReceiveQueue::FIFO_ReceiveQueue ():first (NULL), last (NULL), max_size (MAX_NUMBER_OF_PROCESSES), current_size (0)
{
}


FIFO_ReceiveQueue::FIFO_ReceiveQueue (int size):first (NULL), last (NULL), max_size (size), current_size (0)
{
}

void
FIFO_ReceiveQueue::setFirst (ptFIFO_ReceiveQueueNode Node)
{
  first = Node;
}

void
FIFO_ReceiveQueue::setLast (ptFIFO_ReceiveQueueNode Node)
{
  last = Node;
}

ptFIFO_ReceiveQueueNode
FIFO_ReceiveQueue::getFirst ()
{
  return first;
}

ptFIFO_ReceiveQueueNode
FIFO_ReceiveQueue::getLast ()
{
  return last;
}

int
FIFO_ReceiveQueue::getMax_size ()
{
  return max_size;
}

int
FIFO_ReceiveQueue::getCurrent_size ()
{
  return current_size;
}

void
FIFO_ReceiveQueue::insert (ProcessID_type Process_ID)
{
  if (current_size < max_size)
    {
      ptFIFO_ReceiveQueueNode node;
      node = new FIFO_ReceiveQueueNode (Process_ID);
      if (first == NULL)
	{
	  first = node;
	  last = node;
	}
      else
	{
	  last->setNext (node);
	  last = node;
	}
      current_size++;
    };
}

void
FIFO_ReceiveQueue::remove (ProcessID_type & Process_ID)
{
  if (!(this->empty ()))
    {
      ptFIFO_ReceiveQueueNode node = new FIFO_ReceiveQueueNode ();
      node = first;
      first = node->getNext ();
      if (last == node)
	last = NULL;
      Process_ID = node->getContents ();
      current_size--;
    };
}

bool
FIFO_ReceiveQueue::empty ()
{
  return (first == NULL);
}

bool
FIFO_ReceiveQueue::full ()
{
  return (max_size == current_size);
}

void
FIFO_ReceiveQueue::printFile (char *result_file)
{
  FILE *file_out;
  ptFIFO_ReceiveQueueNode iter = first;

  file_out = fopen (result_file, "wt");
  if (!file_out)
    {
      fprintf (stderr, "Can not open file %s\n", result_file);
      exit (1);
    };
  while (iter != NULL)
    {
      fprintf (file_out, "%d\n", iter->getContents ());
      iter = iter->getNext ();
    };
  fclose (file_out);
}
