module AADL_DATAPORT=

use AADL_TYPE;
use AADL_PROPERTY;
use SIGNALLIB;


process InDataPort_behavior = 
{type item_type;}
(? event Frozen_time_event;
   item_type write_flow;
 ! item_type read_flow;)
  pragmas
       Comment "a Signal process that represents the behavior of an in data port of a thread. The in data is frozen (memorized) at Frozen_time_event, represented by Typed_AT(), and it is memorized by another Typed_AT() for the use"
               "@parameter item_type: type of the write_flow and read_flow"
               "@input Frozen_time_event: event of frozen time"
               "@input write_flow: data write to the in data port"
               "@output read_flow: a memorized value of write_flow when read_flow is present"
     end pragmas
(  | frozen_flow := Typed_AT{item_type}(write_flow, Frozen_time_event) 
   | read_flow := Typed_AT{item_type}(frozen_flow, ^read_flow) 
   | )
   where
     item_type frozen_flow;
   end;




process OutDataPort_behavior = 
{type item_type;}
(? event Output_time_event;
   item_type write_flow;
 ! item_type sent_flow;)
  pragmas
       Comment "a Signal process that represents the behavior of an out data port of a thread. The out data is memorized until the time of Output_time_event, represented by the process Typed_AT()."
               "@parameter item_type: type of the write_flow and sent_flow"
               "@input Output_time_event: event of output time"
               "@input write_flow: data write to the in data port"
               "@output sent_flow: a memorized value of write_flow when sent_flow is present"
     end pragmas
(  | sent_flow := Typed_AT{item_type}(write_flow, Output_time_event) 
   | );


end;
