module fifoLib =

  
  
process fifo_reset =
{type T; integer size; T initVal;}
( ? T input;
    event reset;
  ! T output;
    integer counter; )
(| nextCounter := (0 when readAndWrite when reset)
              default (1 when ^input when reset)
              default (0 when reset)
              default (counter when (readAndWrite ^+ writeOnFull ^+ readOnEmpty))
              default (counter+1 when ^input)
              default (counter-1 when ^output)
              default counter
 | counter := nextCounter $1 init 0
 | syncCounter := counter cell tick
 | readAndWrite := (input ^* output) when (counter>0)
 | writeOnFull := (input ^- output) when (counter=size)
 | readOnEmpty := (output ^- input) when (counter=0)
 | buffer := (input window (size+1)) cell tick
 | output := ((zdefaultVal when (syncCounter=0))
         default (buffer[size-syncCounter] when ^input)
         default (buffer[size+1-syncCounter])) when ^output
 | defaultVal := output
             default (initVal when reset)
             default (buffer[0] when ^input when (syncCounter=size))
             default zdefaultVal
 | zdefaultVal := defaultVal $1 init initVal
 | defaultVal ^= tick
 | tick :=  input ^+ reset ^+ output ^+ counter
 |)
where
  event readAndWrite, writeOnFull, readOnEmpty, tick;
  [size+1] T buffer;
  T defaultVal, zdefaultVal;
  integer nextCounter, syncCounter;
end;
  
  
process fifo =
{type A; integer size; A initVal;}
(? A input;
 ! A output;
   integer counter; )
(| nextCounter := (counter when (readAndWrite ^+ writeOnFull ^+ readOnEmpty))
              default (counter+1 when ^input)
              default (counter-1 when ^output)
              default counter
 | counter := nextCounter $1 init 0
 | syncCounter := counter cell tick
 | readAndWrite := (input ^* output) when (counter>0)
 | writeOnFull := (input ^- output) when (counter=size)
 | readOnEmpty := (output ^- input) when (counter=0)
 | buffer := (input window (size+1)) cell tick
 | output := ((zdefaultVal when (syncCounter=0))
         default (buffer[size-syncCounter] when ^input)
         default (buffer[size+1-syncCounter])) when ^output
 | defaultVal := output default (buffer[0] when ^input when (syncCounter=size)) default zdefaultVal
 | zdefaultVal := defaultVal $1 init initVal
 | defaultVal ^= tick
 | tick := input ^+ output ^+ nextCounter
 |)
where
  event readAndWrite, writeOnFull, readOnEmpty, tick;
  [size+1] A buffer;
  A zdefaultVal, defaultVal;
  integer nextCounter, syncCounter;
end;
  
end;
